# -*- encoding: utf-8 -*-
require 'spec_helper'
require File.expand_path(File.dirname(__FILE__) + '/import')
#インポート処理

describe Import do
  before do
    @t = '{"1": {"a": 1, "b": "Z"}}'
    @j = JSON.parse @t
    @ts = '{"1": {"a": 1, "b": "Z"}, "2": {"a": 2, "b": "Z"}}'
    @js = JSON.parse @ts
    @tes = '{"1": {"a": 0}, "2": {"a": 0}, "3": {"a": 2, "b": "Z"}}'
    @jes = JSON.parse @tes
    @f = File.expand_path(File.dirname(__FILE__) + '/import.json')
  end

  describe '繰り返し処理に於いて' do
    before do
    end
    context '単体データを渡されたとき' do
      it '一回処理' do
        r = []
        Import.each_import @j do |n, i|
          r << i
        end
        r.size.should eq 1
      end
      it '一回処理' do
        r = []
        Import.each_import @j do |n, i|
          r << n
        end
        r.first.should eq "1"
      end
      it '一回処理' do
        r = []
        Import.each_import @j do |n, i|
          r << i
        end
        r.first["a"].should eq 1
        r.first["b"].should eq "Z"
      end
    end
    context '複数を渡されたとき' do
      it '二回処理' do
        r = []
        Import.each_import @js do |n, i|
          r << i
        end
        r.size.should eq 2
      end
    end
  end
  
  describe 'テキスト取り込みに於いて' do
    #成功でTrue、パース失敗でFalse、失敗は保存エラーのモデルを配列で返す
    before do
    end
    context 'つつがなく終わるとき' do
      it 'Json解析を依頼する' do
        JSON.should_receive(:parse_no_except).with(any_args).exactly(1)
        JSON.stub(:parse_no_except).with(any_args).and_return(@j)
        Import.import(@t) {|name, attr| Import.new(attr)}
      end
      it '更新を一回依頼する' do
        JSON.stub(:parse_no_except).with(any_args).and_return(@j)
        Import.any_instance.stub(:valid?).with(any_args).and_return(true)
        #newでスタブを作るとインスタンス生成ができないので、テスト用スタブで
        Import.should_receive(:etest).with(any_args).exactly(1)
        Import.import(@t) {|name, attr| Import.etest ; Import.new(attr) }
      end
      it '[]を返す' do
        JSON.stub(:parse_no_except).with(any_args).and_return(@j)
        Import.any_instance.stub(:valid?).with(any_args).and_return(true)
        Import.import(@t) {|name, attr| Import.new(attr)}.should eq []
      end
    end
    context '複数データがつつがなく終わるとき' do
      it 'ライセンス更新を二回依頼する' do
        JSON.stub(:parse_no_except).with(any_args).and_return(@js)
        Import.any_instance.stub(:valid?).with(any_args).and_return(true)
        #newでスタブを作るとインスタンス生成ができないので、テスト用スタブで
        Import.should_receive(:etest).with(any_args).exactly(2)
        Import.import(@ts) {|name, attr| Import.etest ; Import.new(attr)}.should eq []
      end
      it '[]を返す' do
        Import.import(@ts) {|name, attr| Import.new(attr)}.should eq []
      end
    end
    #例外ケース
    context 'Json解析に失敗したとき' do
      before do
        JSON.stub(:parse_no_except).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        Import.import(@t){|name, attr| Import.new(attr)}.should be_false
      end
    end
    context '作成に失敗したとき' do
      before do
        Import.any_instance.stub(:valid?).with(any_args).and_return(false)
      end
      it '配列を返す' do
        r = Import.import(@t){|name, attr| Import.new(attr)}
        r.is_a?(Array).should be_true
      end
      it '配列の中身は一件' do
        r = Import.import(@t){|name, attr| Import.new(attr)}
        r.should have(1).items
      end
      it 'オブジェクトが入っている' do
        r = Import.import(@t){|name, attr| Import.new(attr)}
        r.first.is_a?(Import).should be_true
      end
    end
    context '複数の作成に失敗したとき' do
      #三件中、二件の失敗、一件を成功させ、成功データは戻り値に含まないことを確認する
      it '途中で保存に失敗しても全件更新依頼する' do
        Import.import(@tes){|name, attr| Import.create(attr)}
      end
      it '配列を返す' do
        r = Import.import(@tes){|name, attr| Import.create(attr)}
        r.is_a?(Array).should be_true
      end
      it '配列の中身は2件' do
        r = Import.import(@tes){|name, attr| Import.create(attr)}
        r.should have(2).items
      end
      it '配列の中身は失敗したオブジェクトが入っている' do
        r = Import.import(@tes){|name, attr| Import.create(attr)}
        r[0].is_a?(Import).should be_true
        r[0]["a"].should eq 0
        r[1].is_a?(Import).should be_true
        r[1]["a"].should eq 0
      end
    end
  end
  
  describe 'インポートエラーの表示に於いて' do
  end
  
  describe 'ファイル取り込みに於いて' do
    before do
      Import.stub(:import).with(any_args).and_return(true)
    end
    context 'つつがなく終わるとき' do
      before do
        Import.stub(:import).with(any_args).and_return(true)
      end
      it 'ファイルを開いてテキストを読む' do
        File.should_receive(:open).with(@f, 'r').exactly(1)
        Import.import_file(@f)
      end
      it 'テキスト取り込みを依頼する' do
        Import.should_receive(:import).with(any_args).exactly(1)
        Import.import_file(@f)
      end
      #テキスト取り込み成功でTrueが返る
      it 'Trueを返す' do
        Import.import_file(@f).should be_true
      end
    end
    context 'ファイルが開けないとき' do
      before do
        File.stub(:open).with(any_args).and_raise('StandardError')
      end
      it 'ファイルエラーのメッセージを出力する' do
        pending
      end
      it 'Falseを返す' do
        Import.import_file(@f).should be_false
      end
    end
    #失敗したときは、失敗したライセンスが配列で返る
    context 'テキスト取り込みが失敗したとき' do
      before do
        Import.stub(:import).with(any_args).and_return(false)
      end
      it '各コモンライセンスのエラーメッセージを出力する' do
        pending
      end
      it 'Falseを返す' do
        Import.import_file(@f).should be_false
      end
    end
  end
  
end

